/*
 *
 *  Computer as Handsfree device
 *
 *  Copyright (C) 2004  Simon Vogl <vogl@soft.uni-linz.ac.at>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation;
 *
 *  Software distributed under the License is distributed on an "AS
 *  IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 *  implied. See the License for the specific language governing
 *  rights and limitations under the License.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <termios.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <time.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/sco.h>
#include <bluetooth/rfcomm.h>

#include <unistd.h>
#include <stdarg.h>     /* we need va_list */

#include "sound.h"

#ifdef OSS
//
// OSS can be defined to send xml events at certain occations
// via a HTTP POST request. 
//
#  include "post.h"
#endif

char *btname;
char telnr[100];

// sound devices:
#ifndef ALSA_PLAY
#define ALSA_PLAY "default"
#endif
#ifndef ALSA_REC
#define ALSA_REC "plughw:0,0"
#endif

#ifdef OSS
char postBuf[10240];

char* prt(char*targ, const char *fmt, ...)
{
    int ret;
    va_list va;
    va_start(va, fmt);

    ret = vsprintf(targ,fmt,va);
    return (targ+ret);
}

void postCmd(const char*cmd) 
{
    int s;
    char *b=postBuf;

    b = prt(b, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
    b = prt(b, "<event source=\"phone\" target=\"system\" id=\"%d\">\n",time(NULL));
    b = prt(b, "<command type=\"%s\">\n",cmd);
    b = prt(b, "<param name=\"type\" value=\"bt\"/>\n");
    b = prt(b, "<param name=\"mac\" value=\"bt:%s\"/>\n",btname);
    if (telnr[0]!='\0') {
	b = prt(b, "<param name=\"tel\" value=\"%s\"/>\n",telnr);
    }
    b = prt(b, "</command>\n");
    b = prt(b,"</event>\n");

    s = init_post("localhost", 2345); 

    fprintf(stderr, "sending\n%s\n",postBuf);

    post_xml(s, "/push", NULL, postBuf );
    read_answer( s );
    close(s);
}


#else 
void postCmd(const char*cmd) {}
#endif


static volatile int terminate = 0;

static void sig_term(int sig) {
    terminate = 1;
}

static void error(const char *fmt, ...)
{
    va_list va;

    va_start(va, fmt);
    fprintf(stderr, "Error: ");
    vfprintf(stderr, fmt, va);
    fprintf(stderr, "\n");
    va_end(va);
}


static int rfcomm_connect(bdaddr_t *src, bdaddr_t *dst, uint8_t channel)
{
    struct sockaddr_rc addr;
    int s;

    if ((s = socket(PF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM)) < 0) {
	return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.rc_family = AF_BLUETOOTH;
    bacpy(&addr.rc_bdaddr, src);
    addr.rc_channel = 0;
    if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	close(s);
	return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.rc_family = AF_BLUETOOTH;
    bacpy(&addr.rc_bdaddr, dst);
    addr.rc_channel = channel;
    if (connect(s, (struct sockaddr *)&addr, sizeof(addr)) < 0 ){
	close(s);
	return -1;
    }

    return s;
}

static int sco_connect(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t *mtu)
{
    struct sockaddr_sco addr;
    struct sco_conninfo conn;
    struct sco_options opts;
    int s, size;

    if ((s = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_SCO)) < 0) {
	perror("socket");
	return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sco_family = AF_BLUETOOTH;
    bacpy(&addr.sco_bdaddr, src);
    if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	perror("bind");
	close(s);
	return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sco_family = AF_BLUETOOTH;
    bacpy(&addr.sco_bdaddr, dst);
    if (connect(s, (struct sockaddr *)&addr, sizeof(addr)) < 0 ){
	perror("connect");
	close(s);
	return -1;
    }

    size = sizeof(conn);
    if (getsockopt(s, SOL_SCO, SCO_CONNINFO, &conn, &size) < 0) {
	perror("getsockopt");
	close(s);
	return -1;
    }

    size = sizeof(opts);
    if (getsockopt(s, SOL_SCO, SCO_OPTIONS, &opts, &size) < 0) {
	perror("getsockopt2");
	close(s);
	return -1;
    }

    if (handle)
	*handle = conn.hci_handle;

    if (mtu)
	*mtu = opts.mtu;

    return s;
}



static int sco_socket_listen(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t *mtu)
{
    struct sockaddr_sco addr;
    int s;

    if ((s = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_SCO)) < 0) {
	return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sco_family = AF_BLUETOOTH;
    bacpy(&addr.sco_bdaddr, src);
    if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	close(s);
	return -1;
    }

    memset(&addr, 0, sizeof(addr));
    addr.sco_family = AF_BLUETOOTH;
    bacpy(&addr.sco_bdaddr, dst);

    if (listen(s, 10) < 0 ){
	close(s);
	return -1;
    }
    printf("listening s set up.\n");
    return s;
}


static int sco_socket_accept(int s, bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t *mtu)
{
    struct sockaddr_sco addr;
    struct sco_conninfo conn;
    struct sco_options opts;
    int size, sco;
    size = sizeof(addr);

    if ( (sco = accept(s, (struct sockaddr *)&addr, &size)) < 0 ){
	perror("accept");
	return -1;
    }

    if (getsockopt(sco, SOL_SCO, SCO_CONNINFO, &conn, &size) < 0) {
	perror("getsockopt 1");
	close(sco);
	return -1;
    }

    size = sizeof(opts);
    if (getsockopt(sco, SOL_SCO, SCO_OPTIONS, &opts, &size) < 0) {
	perror("getsockopt 2");
	close(sco);
	return -1;
    }

    if (handle)
	*handle = conn.hci_handle;

    if (mtu)
	*mtu = opts.mtu;
    fprintf(stderr,"handle %d, mtu %d\n",conn.hci_handle, opts.mtu);
    return sco;
}


/******************* RFCOMM utilities
 *******************/

static void usage(void)
{
    printf("Usage:\n"
	   "\thstest <file> <bdaddr> [channel]\n"
	   "\thstest <file> <bdaddr> [channel]\n");
}

#define PLAY	1
#define RECORD	2

static int writecmd(int rd, char*cmd)
{
    fprintf(stderr,"sending.cmd %s\n",cmd);
    return write(rd, cmd,strlen(cmd));
}

char retbuf[255];
int retCode;

#define RETC_NONE 0 
#define RETC_OK 1 
#define RETC_ERR 2 
#define RETC_RING 3 
#define RETC_RSLT 4  // +blah:
#define RETC_TEXT 4 

#define RET_OK 0
#define RET_ERR -1
#define RET_TIMEOUT -2

int docmd(int rd, char*cmd)
{
    fd_set rfds;
    struct timeval timeout;
    int sel,rlen;

    fprintf(stderr,"[do] doing %s\n",cmd);
    writecmd(rd,cmd);
    fprintf(stderr,"[do] reading\n");
    FD_ZERO(&rfds);
    FD_SET(rd, &rfds);
    timeout.tv_sec = 0;
    timeout.tv_usec = 500000; // half a sec should be enough

    rlen=-1;

    while (rlen<=0 && ((sel = select(rd + 1, &rfds, NULL, NULL, &timeout)) > 0)) {
	if (!FD_ISSET(rd,&rfds)) continue;
	memset(retbuf, 0, sizeof(retbuf));
	rlen = read(rd, retbuf, sizeof(retbuf));
	if (rlen > 0) { 
	    retbuf[rlen]=0;
	    fprintf(stderr,"[do] read: [%s]",retbuf);

	    if (strstr(retbuf,"OK\r\n")>=retbuf) {
		retCode = RETC_OK;
		return RET_OK;
	    }
	    if (strstr(retbuf,"ERROR")>=retbuf) {
		retCode = RETC_ERR;
		return RET_ERR;
	    }
	    if (strstr(retbuf,"RING\r\n")>=retbuf) {
		retCode = RETC_RING;
		return RET_OK;
	    }

	    retCode = RETC_TEXT;
	    return RET_OK;
	} else {
	    //	    return RET_ERR; // ENODATA;
	    fprintf(stderr,".");
	}
    }

    return RET_TIMEOUT;
}


typedef void (*hf_fsm)(int fd, char*buf);

int setup_sco=0;

hf_fsm currentstate;
void state_init(int rd, char*buf);
void state_conn(int rd, char*buf);
void state_answer(int rd, char*buf);

void state_init(int rd, char*buf)
{
    static int phase=0;
    int ok=0, err=0;
    int wlen=0;

    if (strstr(buf,"ERROR")!=NULL) {
	fprintf(stderr, "Error found in phase %d.\n",phase);
	err=1;
    }
    if (strstr(buf,"\r\nOK\r\n")!=NULL) {
	ok=1;
    }
    
    if (phase == 0 ) {
	if (err) {
	    error("0.96 handsfree detected (or you are using the wrong channel!)");
	    err=0;
	    wlen = writecmd(rd, "AT+CIND=?\r"); 
	    phase=1;		
	}
	if (strstr(buf,"+BRSF:") != NULL) { // init link. request indics
	    // parse capabs
	    fprintf(stderr,"boo.\n");
	    wlen = writecmd(rd, "AT+CIND=?\r"); 
	    phase=1;
	}
    } else if (phase == 1) {
	if (strstr(buf,"+CIND:") != NULL) {
	    // parse indicators
	}
	if (ok) {
	    wlen = writecmd(rd, "AT+CIND?\r"); 		
	    phase++;
	}
    } else if (phase == 2) {
	if (strstr(buf,"+CIND:") != NULL) {
	    // parse indicator result
	}
	if (ok) {
	    wlen = writecmd(rd, "AT+CMER=3,0,0,1\r");			    
	    phase++;
	}
    } else if (phase == 3) {
	if (strstr(buf,"+CIND:") != NULL) {
	    // parse CMER result
	}
	if (ok) {
	    wlen = writecmd(rd, "AT+CMER=3,0,0,1\r");			    
	    phase++;
	}
    } else if (phase == 4) {
	if (ok) {
	    wlen = writecmd(rd, "AT+CLIP=1\r");			    
	    phase++;
	}
    } else if (phase == 5) {
	if (err) err=0; // ignore CLIP error

	wlen = writecmd(rd, "AT+NREC=0\r");			    
	phase++;

    } else if (phase == 6) {
	if (err) err=0; // ignore CLIP error

	wlen = writecmd(rd, "AT+CHLD=?\r");			    
	phase++;

    } else if (phase == 7) {
	err=0; // ignore clip errors;
	//	wlen = writecmd(rd, "AT+CLIP=1\r");			    
	
	currentstate = state_conn;

	setup_sco=1; // try to connect SCO

	postCmd("handsfree_up");

	phase=0; // reset for re-entry
    }
}

void state_conn(int rd, char*buf)
{
    char *str;
    int ok=0, err=0, ring=0;
    //int wlen=0;

    if (strstr(buf,"ERROR")!=NULL) {
	fprintf(stderr, "Error\n");
	err=1;
    }
    if (strstr(buf,"\r\nOK\r\n")!=NULL) {
	ok=1;
    }

    if (strstr(buf,"RING") != NULL ) {
	ring=1;
    } 
    if ( (str=  strstr(buf,"+CLIP:")) != NULL) {
	char event[40];
	char *num = telnr;
	str+=8; // go after \"
	fprintf(stderr,">>str:%s\n",str);
	while (*str && *str!='"') {
	    *num = *str;
	    num++; 
	    str++;
	}
	*num = '\0'; // but don't post...
	fprintf(stderr,">>telnr:%s\n",telnr);

	sprintf(event,"incoming");
	postCmd(event);
    
    } else if ( (str=  strstr(buf,"+CIEV:")) != NULL) {
	int which,what;
	char *names[] = { "???" , "service","call","BTcall_setup" ,"callsetup" };
	char event[40];

	// call progress
	str+=7; // skip the [+ciev: ]
	which=*str-'0';
	what = *(str+2)-'0';
	fprintf(stderr, "event indication: %s is %d\n", names[which], what);

	if (which==3 && what==1) {
	    setup_sco=1;
	}
	if (which==1) {
	    if (what) {
		sprintf(event,"connected");
	    } else {
		sprintf(event,"disconnected");
	    }
	    postCmd(event);
	}
	
    } else {
	if (!ok)
	    fprintf(stderr,"can't grok [%s]", buf);
    }
    if (err) {
	error("Error in state_conn - ignoring");
	//currentstate = state_init; // start over
    }
    if (ring) {
	postCmd("ring");
    }
}

// read answer for AT command. only available from state_conn
void state_answer(int rd, char*buf)
{
    int ok=0, err=0, ring=0;
    //int wlen=0;

    if (strstr(buf,"ERROR")!=NULL) {
	fprintf(stderr, "Error\n");
	err=1;
    }
    if (strstr(buf,"\r\nOK\r\n")!=NULL) {
	ok=1;
    }

    if (strstr(buf,"RING") != NULL ) {
	ring=1;
	//fprintf(stderr, "taking call\n");
	//wlen = writecmd(rd, "ATA\r\n"); // the hs doesn't answer
    } 
    if (ok || err ) {
	currentstate = state_conn; // start over	
    }
}
void analyse(char*, int);
void prtb(char*, int);
void reset_bins();
void print_bins();
int  check_bins();

int main(int argc, char *argv[])
{
    struct sigaction sa;

    fd_set rfds;
    //fd_set wfds;
    struct timeval timeout;
    unsigned char buf[2048];
    int maxfd, sel, rlen, wlen, fdout, fdin, ctrl;
    int pb,rec;

    unsigned char ctrlLine[256]; // reading from stdin...
    int ctrlPos=0;

    bdaddr_t local;
    bdaddr_t bdaddr;
    bdaddr_t bdany;
    uint8_t channel;

    char *filename;
    mode_t filemode;

    int dd, rd, sd, fd, scolink=0, sco_adj=0, sco_count=0;
    uint16_t sco_handle, sco_mtu, vs;

    int state=0;
    reset_bins();
    currentstate=state_init;
    telnr[0]=0;

    init_sound_split(ALSA_PLAY,ALSA_REC);

    switch (argc) {
    case 3:
	str2ba(argv[1], &bdaddr);
	channel = 3;
	break;
    case 4:
	str2ba(argv[1], &bdaddr);
	channel = atoi(argv[2]);
	break;
    default:
	usage();
	exit(-1);
    }

    btname = argv[2];

    filemode = O_RDONLY;
    filename = "";//argv[1];
    /** set up bt stack */

    hci_devba(0, &local);
    dd = hci_open_dev(0);

    hci_read_voice_setting(dd, &vs, 1000);
    vs = htobs(vs);
    fprintf(stderr, "Voice setting: 0x%04x\n", vs); 
    close(dd);

    if (vs != 0x0060) {
	fprintf(stderr, "The voice setting must be 0x0060\n");
	return -1;
    }

    fd = -1;

    /*
    fdout = open("out.raw",O_WRONLY | O_CREAT | O_TRUNC);
    if (fdout<0) { perror("could not save to output!"); };
    */
    fdout=-1;

    memset(&sa, 0, sizeof(sa));
    sa.sa_flags = SA_NOCLDSTOP;
    sa.sa_handler = sig_term;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT,  &sa, NULL);

    sa.sa_handler = SIG_IGN;
    sigaction(SIGCHLD, &sa, NULL);
    sigaction(SIGPIPE, &sa, NULL);


    if ((rd = rfcomm_connect(&local, &bdaddr, channel)) < 0) {
	perror("Can't connect RFCOMM channel");
	return -1;
    }

    fprintf(stderr, "RFCOMM channel connected\n");

    sd=0;
#if 0 /* SCO connection is set up by the audio gateway */
    fprintf(stderr, "Setting up SCO channel connected\n");
    if ((sd = sco_connect(&local, &bdaddr, &sco_handle, &sco_mtu)) < 0) {
	perror("Can't connect SCO audio channel");
	close(rd);
	return -1;
    }
    fprintf(stderr, "SCO audio channel connected (handle %d, mtu %d)\n", sco_handle, sco_mtu);
#endif
    //#else
#if 0
    if ((sd = sco_socket_listen(&local, &bdany, &sco_handle, &sco_mtu)) < 0) {
	fprintf(stderr, "Can't init listening SCO audio channel");
	close(rd);
	return -1;
    }
#endif

    //    wlen = writecmd(rd, "AT+CKPD=200\r"); // request audio link

    wlen = writecmd(rd, "AT+BRSF=31\r"); // request audio link

    maxfd = (rd > sd) ? rd : sd;

    // sound devices...
    pb = get_pb_dev();
    rec = get_rec_dev();

    scolink=0;
    state=0;
    //    ctrl = 0; // control pipe reads from stdin
    ctrl = open("/tmp/ctrl",O_RDWR);
    if (ctrl==-1) {
	perror("opening control pipe - set to stdin");
	ctrl=0;
    }
    while (!terminate) {

	maxfd=rd;

	//no write checking.
	//FD_ZERO(&wfds);
	//FD_SET(pb, &wfds);

	FD_ZERO(&rfds);
	FD_SET(ctrl, &rfds);
	FD_SET(rd, &rfds);

	if (sd>0) 
	    FD_SET(sd, &rfds);

	if (scolink>0) {
	    FD_SET(scolink, &rfds);
	    //FD_SET(scolink, &wfds);

	    //FD_SET(rec, &rfds);
	    //if (maxfd<rec) maxfd=rec;
	}

	//maxfd:
	if (maxfd<sd) maxfd=sd;
	if (maxfd<scolink) maxfd=scolink;

	timeout.tv_sec = 0;
	timeout.tv_usec = 1000000;

	//if ((sel = select(maxfd + 1, &rfds, &wfds, NULL, &timeout)) > 0) {
	if ((sel = select(maxfd + 1, &rfds, NULL, NULL, &timeout)) > 0) {
	    //fprintf(stderr,".");
	    /*        
	    if ( FD_ISSET(pb, &wfds) ) {
		//fprintf(stderr,".");
		sound_play(buf, 48);
		
	    }
	    */
	    if (scolink>0 && FD_ISSET(scolink, &rfds) ) {
		rlen = read(scolink, (buf+sco_adj), sizeof(buf)-2);

		sco_count++;
		if (sco_count<80 ) {
		    analyse(buf+sco_adj,rlen);
		    if (sco_count==1) {
			prtb(buf+sco_adj,rlen);
		    }
		} else if (sco_count < 80) {
		    analyse(buf,rlen);
		    if (sco_count==1) {
			prtb(buf+sco_adj,rlen);
		    }
		}
		if (sco_count == 40) {
		    print_bins();
		    if (count_bins()>48) {
			sco_adj = ! check_bins();
			fprintf(stderr,"adj=%d\n",sco_adj);
			reset_bins();
		    } else {
			fprintf(stderr,"silence..\n");
		    }
		}
		if (sco_count == 80) {
		    print_bins();
		    //sco_adj = ! check_bins();
		    //fprintf(stderr,"adj=%d\n",sco_adj);
		}


		if (sco_adj) {
		    buf[0] = buf[rlen];
		}

		if (rlen > 0) {
		    sound_play(buf,rlen);

		    if (fdout > 0) { // save audio data
			wlen = write(fdout, buf, rlen);
		    }
		    if (fd>0) { // play file
			int r;
			r=read(fd, buf, 48);
			write(scolink, buf, rlen);
			if (r<0) {
			    close(fd); 
			    fd=0;
			}
		    } else {
			rlen = sound_rec(buf,48);
			write(scolink, buf, rlen);
			if (fdin>0) { // save recorded voice
			    write(fdin, buf, rlen);  
			}
		    }
		} else if (rlen<0) {
		    fprintf(stderr,"lost sco audio connection (%d %d)!\n",rlen,errno);
		    close(scolink);
		    scolink=0;

		    sound_pause();

		    if ( !sd && (sd = sco_socket_listen(&local, &bdany, &sco_handle, &sco_mtu)) < 0) {
			fprintf(stderr, "Can't init listening SCO audio channel");
			sd = 0;
		    } else {
			continue;
		    }
	        }
	    }

	    if (FD_ISSET(rd, &rfds)) {
		memset(buf, 0, sizeof(buf));
		rlen = read(rd, buf, sizeof(buf));
		if (rlen > 0) {
		    fprintf(stderr, ">>%s\n", buf);		    
		    //hf_fsm laststate = currentstate;

		    currentstate(rd, buf); 

		    //		    if (laststate == state_init && currentstate == state_conn) {
		    if (!scolink && setup_sco) { 
			if (sd) { close(sd); sd=0;}
			// end of init phase - make audio connection
			//usleep(10000);
			if ((scolink = sco_connect(&local, &bdaddr, &sco_handle, &sco_mtu)) < 0) {
			    char nam[18];
			    ba2str(&bdaddr,nam);
			    error("Can't connect SCO audio channel @ %s",nam);
			    scolink=0;
			    //close(rd);
			    //return -1;
			} else {
			    char buf[64];
			    memset(buf,0,64);
			    write(scolink,buf,48);

			    sound_resume();
			}
		    }
		    setup_sco=0;
		} else if (rlen<0) {
		    fprintf(stderr,"RFCOMM link terminated!\n");
		    terminate=1;
		}
	    }

	    if (FD_ISSET(ctrl, &rfds)) { //stdin		
		int i;
		int lf=0;

		rlen = read(ctrl, ctrlLine+ctrlPos, sizeof(ctrlLine-ctrlPos));

		for (i=0;i<rlen;i++) {
		    if (ctrlLine[ctrlPos+i] == '\n' || ctrlLine[ctrlPos+i] == '\r' ) {
			lf =1;
			break;
		    }
		}
		ctrlPos += i; // set marker 
		if (!lf) {
		    continue;
		}
		ctrlLine[ctrlPos]='\0';
		//fprintf(stderr, "\n>>>parsing %d bytes %s\n", ctrlPos, ctrlLine);
		ctrlPos=0;

		if (ctrlLine[0]=='h' || ctrlLine[0]=='H') {
		    //hangup
		    currentstate = state_answer;
		    wlen = writecmd(rd, "AT+CHUP\r");
		} else if (ctrlLine[0]=='a' || ctrlLine[0]=='A') {
		    //answer
		    currentstate = state_answer;
		    wlen = writecmd(rd, "ATA\r");
		} else if (ctrlLine[0]=='S') {
		    //answer
		    sco_count=0;
		} else if (ctrlLine[0]=='d' || ctrlLine[0]=='D') {
		    //dial - number follows
		    char b[80];
		    currentstate = state_conn;
		    sprintf(b,"ATD%s\r", ctrlLine+1);
		    wlen = writecmd(rd, b);
		} else if (ctrlLine[0]=='o' ) { 
		    if (fdout<0) {
			char *name = "out.raw";
			if (ctrlLine[1])
			    name = ctrlLine+1;
			fdout = open(name,O_WRONLY | O_CREAT | O_TRUNC);
			fprintf(stderr,"openend %s for sound output monitoring\n",name);
		    } else {
			close(fdout);
			fdout=-1;
		    }
		} else if (ctrlLine[0]=='i' ) { 
		    if (fdout<0) {
			char *name = "in.raw";
			if (ctrlLine[1])
			    name = ctrlLine+1;
			fdin = open(name,O_WRONLY | O_CREAT | O_TRUNC);
			fprintf(stderr,"openend %s for record monitoring\n",name);
		    } else {
			close(fdout);
			fdout=-1;
		    }
		} else if (ctrlLine[0]=='X' ) { 
		    sco_adj=1-sco_adj;
		    fprintf(stderr,"offset %d\n",sco_adj);
		} else if (ctrlLine[0]=='f' ) { 
		    // open file and play :)
		    if ((fd = open(ctrlLine+1,O_RDONLY )) < 0) {
			error("Can't open input file %s: %s",ctrlLine+1,strerror(errno));fflush(stderr);
			fd=-1;
		    }
		} else if (ctrlLine[0]=='t' ) { 
		    // terminate playback
		    if (fd>0) {
			close(fd);
			fd=-1;
		    }
		    //send raw command to modem >:)
		    char b[80];
		    currentstate = state_answer;
		    sprintf(b,"%s\r", ctrlLine+1);
		    wlen = writecmd(rd, b);
		}
		ctrlLine[0]='\0'; // reset.

		// nokia needs:
		//   wlen = writecmd(rd, "AT+VGM=15\r"); // the hs doesn't answer	
		//    wlen = writecmd(rd, "AT+VGS=15\r"); // the hs doesn't answer
	    }

	    if (sd>0 && FD_ISSET(sd, &rfds)) { // master control found
		fprintf(stderr,"sd event\n");
		if (scolink) {
		    fprintf(stderr,"Warning! SCO link already set up!!\n");
		}
		// todo: skip init when set up 
		scolink = sco_socket_accept(sd, &local, &bdaddr, &sco_handle, &sco_mtu);
		if (scolink >0) {
		    fprintf(stderr,"Houston, we was connected! %d %d\n",scolink, maxfd);
		    write(scolink,buf,48);
		    sco_count=0;sco_adj=0;reset_bins();
		    sound_resume();
		} else {
		    fprintf(stderr,"Houston, we have a problem!\n");
		    scolink=0;
		}
	    }
	}
    }

    close(sd);
    sleep(1);
    close(rd);
    close(fdout);
    close(fd);
    if (scolink) close(scolink);

    exit_sound();

    postCmd("handsfree_down");
	
    return 0;
}


int bins[16];
int indicator;

void reset_bins()
{
    int i;
    indicator=0;
    for (i=0;i<16;i++) {
	bins[i]=0;
    }
}

void print_bins()
{
    int i;
    for (i=0;i<16;i++) {
	fprintf(stderr," %4d",bins[i]);
    }
    fprintf(stderr,"\n");
}


/** checks if the energy is concentrated in the middle 
 */

int check_bins()
{
    int i;
    int sum=0,middle=0;
    for (i=0;i<16;i++) {
	sum+=bins[i];
    }
    for (i=4;i<12;i++) {
	middle+=bins[i];
    }
    fprintf (stderr,"indicator %d\n",indicator);
    if (indicator>20) return 1;

    fprintf (stderr, "check: total %d, middle %d, fraction %d %% val %d\n",sum,middle, (middle*100/sum),(middle > sum*0.8) );
    return (middle > sum*0.8);
}

int count_bins()
{
    int i;
    int sum=0;
    for (i=0;i<16;i++) {
	sum+=bins[i];
    }
    return (sum);
}

void analyse(char* buf, int len)
{
    int i;
    short *ptr;

    if (len != 48) return;

    ptr = (short*)buf;

    for (i=0 ; i<24 ; i++) {
	short i = *ptr++;
	if  (( i && 0x00ff) == 0x00ff) // shifted -1..-f
	     indicator++;
	i /= 256; 
	i /= 16;
	i += 8; 
	bins[i]++;
    }
}

void prtb(char* buf, int len)
{
    int i;
    short *ptr;

    if (len != 48) return;

    ptr = (short*)buf;

    for (i=0 ; i<24 ; i++) {
	short i = *ptr++;
	fprintf(stderr,"%04hx ",i);
    }
    fprintf(stderr,"\n");
    ptr = (short*)buf;
    for (i=0 ; i<24 ; i++) {
	short i = *ptr++;
	fprintf(stderr,"%4hd ",i);
    }
    fprintf(stderr,"\n");
}
