/*
 *
 *  Computer as Headset utility
 *
 *  Copyright (C) 2004  Simon Vogl <vogl@soft.uni-linz.ac.at>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation;
 *
 *  Software distributed under the License is distributed on an "AS
 *  IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 *  implied. See the License for the specific language governing
 *  rights and limitations under the License.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <termios.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <unistd.h>
#include <stdarg.h>     /* we need va_list */

/*sound card*/
#include <alsa/asoundlib.h>
#include <alsa/pcm.h>

#ifdef SOLO
static volatile int terminate = 0;

static void sig_term(int sig) {
	terminate = 1;
}
#endif

static void error(const char *fmt, ...)
{
    va_list va;

    va_start(va, fmt);
    fprintf(stderr, "Error: ");
    vfprintf(stderr, fmt, va);
    fprintf(stderr, "\n");
    va_end(va);
}



char *device = "plughw:0,0";			/* playback device */
snd_pcm_format_t format = SND_PCM_FORMAT_S16;	/* sample format */
unsigned int rate = 8000;			/* stream rate */
unsigned int channels = 1;			/* count of channels */
unsigned int buffer_time = 500000;		/* ring buffer length in us */
unsigned int period_time = 100000;		/* period time in us */
double freq = 440;				/* sinusoidal wave frequency in Hz */

snd_pcm_sframes_t buffer_size;
snd_pcm_sframes_t period_size;
snd_output_t *output = NULL;


void set_parameters (snd_pcm_t *hdl)
{
    int err,dir;
    snd_pcm_hw_params_t *hw_params;
			   
    if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
	error ( "cannot allocate hardware parameter structure (%s)\n",
		 snd_strerror (err));
	exit (1);
    }
				 
    if ((err = snd_pcm_hw_params_any (hdl, hw_params)) < 0) {
	error("cannot initialize hardware parameter structure (%s)\n",
		 snd_strerror (err));
	exit (1);
    }
	
    if ((err = snd_pcm_hw_params_set_access (hdl, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
	error("cannot set access type (%s)\n", snd_strerror (err));
	exit (1);
    }
	
    if ((err = snd_pcm_hw_params_set_format (hdl, hw_params, SND_PCM_FORMAT_S16_LE)) < 0) {
	error("cannot set sample format (%s)\n", snd_strerror (err));
	exit (1);
    }
	
    if ((err = snd_pcm_hw_params_set_rate (hdl, hw_params, 8000, 0)) < 0) {
	error("cannot set sample rate (%s)\n", snd_strerror (err));
	exit (1);
    }
	
    if ((err = snd_pcm_hw_params_set_channels (hdl, hw_params, 1)) < 0) {
	error("(nonfatal) cannot set channel count (%s)\n", snd_strerror (err));
	//exit (1);
    }
	
    if ((err = snd_pcm_hw_params (hdl, hw_params)) < 0) {
	error("cannot set parameters (%s)\n", snd_strerror (err));
	exit (1);
    }
	
    /* set the buffer time */
    
    err = snd_pcm_hw_params_set_buffer_time_near(hdl, hw_params, &buffer_time, &dir);

    if (err < 0) {
	error("Unable to set buffer time %i for playback: %s\n", buffer_time, 
	      snd_strerror(err));
	exit(1);
    }
    
    snd_pcm_hw_params_free (hw_params);
}

void open_capture (char *device, snd_pcm_t **capture_handle)
{
    int err;
	
    if ((err = snd_pcm_open (capture_handle, device, SND_PCM_STREAM_CAPTURE, 0)) < 0) {
	error ( "cannot open audio device %s (%s)\n", device, snd_strerror (err));
	exit (1);
    }

    set_parameters(*capture_handle);
	
    if ((err = snd_pcm_prepare (*capture_handle)) < 0) {
	error("cannot prepare audio interface for use (%s)\n",
		 snd_strerror (err));
	exit (1);
    }	
}

void open_playback(char *device, snd_pcm_t **pb_hdl)
{
    int err;
    
    //if ((err = snd_pcm_open (playback_handle, device, SND_PCM_STREAM_PLAYBACK,0)) < 0) {
    if ((err = snd_pcm_open (pb_hdl, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
	error("cannot open audio device %s (%s)\n", 
	      device, snd_strerror (err));
	exit (1);
    }
		   
    set_parameters(*pb_hdl);
       


    if ((err = snd_pcm_prepare (*pb_hdl)) < 0) {
	error("cannot prepare audio interface for use (%s)\n",
	      snd_strerror (err));
	exit (1);
    }
}


snd_pcm_t *aplay;
snd_pcm_t *acapt;

void init_sound(char *device)
{
    open_playback(device, &aplay);
    open_capture(device, &acapt);

}

void init_sound_split(char *devicepb, char *devicerec)
{
    open_playback(devicepb, &aplay);
    open_capture(devicerec, &acapt);

}

void exit_sound()
{
    snd_pcm_close (aplay);
    snd_pcm_close (acapt);
}

void sound_play(char*buf, int len)
{
    int err;
    int state;
    long frames;

    static int i=0;
    static int init=0;
    static long maxbuf=0;
    static int skip_ints=0;

    if (len!=48) fprintf(stderr,"len=%-5d\n",len);
    len/=2;
    state = snd_pcm_state(aplay);
    //fprintf(stderr,"!%d",state);

    if (!init) {
	maxbuf = snd_pcm_avail_update  (   aplay ) ;
	fprintf(stderr,"maxbuf: %ld\n",maxbuf);
	init=1;
    }
    // underrun?
    if (state == SND_PCM_STATE_XRUN) {
	snd_pcm_prepare(aplay);
    }
    // debug...
    frames = snd_pcm_avail_update  (   aplay ) ;
    if (i%10==0) {
      /*
      fprintf(stderr,".%8ld\r",(maxbuf-frames));
	if (i==100) {
	    fprintf(stderr,"\n");
	    i=0;
	}
      */
    }
    
    if ( (maxbuf - frames) > 24*12) {
	// growing delay, drop packet
	skip_ints+=2;
	//fprintf(stderr,"+skip = %d\n",skip_ints);
	//return;
    } else if ( (maxbuf - frames) < 24*2) {
	// beware of underrun -> duplicate
	err = snd_pcm_writei (aplay, buf, len);
	skip_ints=0;
	//fprintf(stderr,"-skip = %d\n",skip_ints);
    } else {
	//fprintf(stderr,".skip = %d\n",skip_ints);
	//fprintf(stderr,".");
    }
    
    i++;
    /* add rate adaption code here: ask for current buffer len.
     * if <min ->
     */
    if (skip_ints>0) 
	len-=skip_ints;
    if ((err = snd_pcm_writei (aplay, buf, len)) != len) {
	if (err == -EPIPE) { // buffer underrun -> duplicate...
	    snd_pcm_prepare(aplay);
	    err = snd_pcm_writei (aplay, buf, len);
	    fprintf(stderr,".run\n");
	}
	if (err != -EPIPE) {
	    fprintf (stderr, "write to audio interface failed (%s)\n",
		 snd_strerror (err));
	}
    }
    //if (state != SND_PCM_STATE_RUNNING)
    //	snd_pcm_start(aplay);
}

int sound_rec(char*buf, int len)
{
    int err;
    len/=2;
    if ((err = snd_pcm_readi (acapt, buf, len)) != len) {
	if (err == -EPIPE) {
	    snd_pcm_prepare(acapt);
	    err = snd_pcm_readi (acapt, buf, len);
	}
	if (err<0 && err != -EPIPE) {
	    fprintf (stderr, "read from audio interface failed (%s)\n",
		     snd_strerror (err));
	}
    }
    //printf("%d\r",*(short*)buf);
    return err*2;
}


void sound_pause()
{
    //snd_pcm_pause(aplay,1);
    snd_pcm_pause(acapt,1);
}


void sound_resume()
{
    //snd_pcm_pause(aplay,0);
    snd_pcm_pause(acapt,0);
}


int get_poll_dev(snd_pcm_t *hdl) 
{
    struct pollfd *ufds;
    int fd, err=-1, count;

    count = snd_pcm_poll_descriptors_count (hdl);
    if (count <= 0) {
	printf("Invalid poll descriptors count\n");
	return count;
    }

    ufds = malloc(sizeof(struct pollfd) * count);
    if (ufds == NULL) {
	printf("No enough memory\n");
	return err;
    }
    printf("poll descriptors count %d\n",count);
    if ((err = snd_pcm_poll_descriptors(hdl, ufds, count)) < 0) {
	printf("Unable to obtain poll descriptors for playback: %s\n", snd_strerror(err));
	return err;
    }
    fd = ufds->fd;
    free(ufds);   
    return fd;
}

int get_pb_dev() 
{
    return get_poll_dev(aplay);
}

int get_rec_dev() 
{
    return get_poll_dev(acapt);
}

#ifdef SOLO
/******************* TEST CODE
 *******************/

static void usage(void)
{
    printf("Usage:\n"
	   "\tsound  <file>\n"
}

int main(int argc, char *argv[])
{
    struct sigaction sa;

    fd_set rfds;
    struct timeval timeout;
    unsigned char buf[2048];
    short ibuf[48],iptr=(short*)buf;
    int maxfd, sel, rlen, wlen, ret,fdout=0;

    char *filename;
    mode_t filemode;
    int mode = 0;
    int dd, rd, sd, fd, infd=1;

    int state=0;

    switch (argc) {
    case 2:
	break;
    default:
	usage();
	exit(-1);
    }

    filemode = O_RDONLY;

    filename = argv[1];

    memset(&sa, 0, sizeof(sa));
    sa.sa_flags = SA_NOCLDSTOP;
    sa.sa_handler = sig_term;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT,  &sa, NULL);

    sa.sa_handler = SIG_IGN;
    sigaction(SIGCHLD, &sa, NULL);
    sigaction(SIGPIPE, &sa, NULL);


    init_sound("default");

    fd = open(filename, filemode);
    for (ret=0;ret<256;ret++) {
	buf[ret] = 0;
    }

    ret=48;

    while (!terminate) {
	ret = read(fd,buf,48);
	sound_play(ibuf,ret/2); // pass # samples!

    }

    exit_sound();
    close(fd);

    return 0;
}
#endif
